'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _constants = require('./constants');

exports.default = new Proxy({}, {
    get: function get(target, name) {
        return find(name);
    }
});


var isNumber = function isNumber(token) {
    return _constants.numbers[token] !== undefined;
};
var isMultiplier = function isMultiplier(token) {
    return _constants.multipliers[token] !== undefined;
};
var isUnit = function isUnit(token) {
    return _constants.units[token] !== undefined;
};

function parse(token) {
    if (isNumber(token)) {
        return { type: 'number', value: _constants.numbers[token] };
    }
    if (isMultiplier(token)) {
        return { type: 'multiplier', value: _constants.multipliers[token] };
    }
    if (isUnit(token)) {
        return { type: 'unit', value: _constants.units[token] };
    }
    throw new Error('Could not parse expression ' + token + '.');
}

function find(name) {
    var terms = name.split('_').map(function (term) {
        return term.toUpperCase();
    }).filter(function (term) {
        return term !== 'AND';
    }).map(parse);

    var sum = 0;
    var partSum = 0;

    for (var i = 0; i < terms.length; i++) {
        var term = terms[i];

        switch (term.type) {
            case 'number':
                {
                    partSum += term.value;
                    break;
                }
            case 'multiplier':
                {
                    partSum = (partSum || 1) * term.value;
                    break;
                }
            case 'unit':
                {
                    sum = sum + partSum * term.value;
                    partSum = 0;
                    break;
                }
        }
    }

    return partSum + sum;
}